//BOI 2012 - Tiny Checker
//
//Takes 4 arguments: input file name, output file name, checker verdict file name (where the checker will store the verdict) and maximum count of pieces.
//The reason for it is that the score is relative to the best count achieved in the contest amongst the contestants.
//During the contest it was maximum possible count (and the contestants would get the guaranteed score, but if no full solutions were submitted, that score could become larger)
//and then changed to actual maximum achieved count.
//
//During the onsite of BOI 2012 the maximum achieved count of pieces for all five tests were 1000, 5000, 1030, 50000, 50001.
//
//Returns 1 if something is wrong. Otherwise returns 0 and a checker verdict in a file, which name is determined by the third argument.
//The file contains the score for the test in the *.?? format.
//Additionally, checker writes the bonus information in stdout, such as what piece doesn’t fit or if all pieces are placed correctly.

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <algorithm>
#define MAXLEN 1000000
#define MAXN 100005

using namespace std;

FILE *fi = NULL;
int N;
static char s[MAXLEN];

int ReadRow () //-1: incorrect, 0: empty row: x: a number
{
	char *stat = fgets(s, MAXLEN, fi);
	if (stat == NULL) return 0;
	int len = strlen(s);
	if (len >= MAXLEN)
	{
		return -1;
	}
	int i;
	char dig = 0;
	for (i = 0; i < len; i++)
	{
		if (s[i] > ' ')
		{
			if (dig != 0) return -1;
			dig = s[i];
		}
	}
	if ((dig >= '1') && (dig <= '9')) return (int)(dig - '0');
	else return -1;
}

int main (int argc, char **argv)
{
	if (argc != 5)
	{
		printf("Incorrect usage of checker");
		return 1;
	}
	FILE *f = NULL, *fo = NULL;
	f = fopen(argv[1],"r");
	if(!f)
	{
		printf("Can't open input file");
		return 1;
	}
	fi = fopen(argv[2],"r");
	if(!fi)
	{
		printf("Can't open output file");
		return 1;
	}
	fo = fopen(argv[3],"w");
	if(!fo)
	{
		printf("Can't open result file");
		return 1;
	}
	int N;
	fscanf(f,"%d",&N);
	static int data[MAXN];
	int i;
	for (i = 0; i < N; i++) fscanf(f,"%d",&(data[i]));
	int MAX;
	sscanf(argv[4],"%d",&MAX);
	static char a[9][9];
	memset(a,0,sizeof(a));
	int cnt = 0;
	int st;
	char err = 0;
	while (cnt < N)
	{
		st = ReadRow();
		if (st <= 0)
		{
			printf("Row %d is not valid",cnt+1);
			err = 1;
			break;
		}
		if ((((data[cnt] == 3) || (data[cnt] >= 6)) && (st >= 9)) || ((data[cnt] == 5) && (st >= 8))) break;
		st--;
		if (data[cnt] == 1)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
		}
		else if (data[cnt] == 2)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[7][st]) break;
			a[7][st] = 2;
		}
		else if (data[cnt] == 3)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[8][st+1]) break;
			a[8][st+1] = 2;
		}
		else if (data[cnt] == 4)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[7][st]) break;
			a[7][st] = 2;
			if (a[6][st]) break;
			a[6][st] = 2;
		}
		else if (data[cnt] == 5)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[8][st+1]) break;
			a[8][st+1] = 2;
			if (a[8][st+2]) break;
			a[8][st+2] = 2;
		}
		else if (data[cnt] == 6)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[7][st]) break;
			a[7][st] = 2;
			if (a[7][st+1]) break;
			a[7][st+1] = 2;
		}
		else if (data[cnt] == 7)
		{
			if (a[8][st+1]) break;
			a[8][st+1] = 2;
			if (a[7][st]) break;
			a[7][st] = 2;
			if (a[7][st+1]) break;
			a[7][st+1] = 2;
		}
		else if (data[cnt] == 8)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[7][st]) break;
			a[7][st] = 2;
			if (a[8][st+1]) break;
			a[8][st+1] = 2;
		}
		else if (data[cnt] == 9)
		{
			if (a[8][st]) break;
			a[8][st] = 2;
			if (a[8][st+1]) break;
			a[8][st+1] = 2;
			if (a[7][st+1]) break;
			a[7][st+1] = 2;
		}
		cnt++;
		int j;
		
		int A, B;
		int drop = 10;
		int temp;
		for (j = 0; j < 9; j++)
		{
			A = -1;
			for (i = 8; i >= 0; i--)
			{
				if (a[i][j] == 2) A = i;
			}
			if (A != -1)
			{
				B = -1;
				for (i = 0; i < 9; i++)
				{
					if (a[i][j] == 1) B = i;
				}
				temp = A - B - 1;
				if (temp < drop) drop = temp;
			}
		}
		for (i = 0; i < 9; i++)
		{
			for (j = 0; j < 9; j++)
			{
				if (a[i][j] == 2)
				{
					a[i][j] = 0;
					a[i-drop][j] = 1;
				}
			}
		}

		i = 0;
		while (i < 9)
		{
			char line = 1;
			for (j = 0; j < 9; j++)
			{
				if (a[i][j] == 0)
				{
					line = 0;
					break;
				}
			}
			if (line)
			{
				for (j = 0; j < 9; j++) a[i][j] = 0;
				int x, y;
				for (x = i+1; x < 9; x++)
				{
					for (y = 0; y < 9; y++)
					{
						a[x-1][y] = a[x][y];
						a[x][y] = 0;
					}
				}
			}
			else i++;
		}
	}
	if ((cnt < N) && (!err))
	{
		printf("Piece %d doesn't fit",cnt+1);
	}
	else if (cnt == N)
	{
		printf("All pieces are placed");
	}
	int res = (2000 * cnt + (MAX / 2)) / MAX;
	fprintf(fo,"%d.%02d\n", res / 100, res % 100);
	fclose(f);
	fclose(fi);
	fclose(fo);
	return 0;
}
